/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or (at
 *  your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *  MA 02110-1301, USA.
 *
 *  Copyright © 2006 Pierre Habouzit
 */
/*
 * Copyright notice from original mutt:
 * Copyright (C) 1996-2000 Michael R. Elkins <me@mutt.org>
 * Copyright (C) 1999-2000 Thomas Roessler <roessler@does-not-exist.org>
 *
 * This file is part of mutt-ng, see http://www.muttng.org/.
 * It's licensed under the GNU General Public License,
 * please see the file GPL in the top level source directory.
 */

#if HAVE_CONFIG_H
# include "config.h"
#endif

#include <stdlib.h>
#ifdef HAVE_SYSEXITS_H
#include <sysexits.h>
#else /* Make sure EX_OK is defined <philiph@pobox.com> */
#define EX_OK 0
#endif

#include <lib-lib/lib-lib.h>

#include "exit.h"

static const struct {
    int v;
    const char *str;
} sysexits_h[] = {
#ifdef EX_USAGE
    { 0xff & EX_USAGE, "Bad usage." },
#endif
#ifdef EX_DATAERR
    { 0xff & EX_DATAERR, "Data format error." },
#endif
#ifdef EX_NOINPUT
    { 0xff & EX_NOINPUT, "Cannot open input." },
#endif
#ifdef EX_NOUSER
    { 0xff & EX_NOUSER, "User unknown." },
#endif
#ifdef EX_NOHOST
    { 0xff & EX_NOHOST, "Host unknown." },
#endif
#ifdef EX_UNAVAILABLE
    { 0xff & EX_UNAVAILABLE, "Service unavailable." },
#endif
#ifdef EX_SOFTWARE
    { 0xff & EX_SOFTWARE, "Internal error." },
#endif
#ifdef EX_OSERR
    { 0xff & EX_OSERR, "Operating system error." },
#endif
#ifdef EX_OSFILE
    { 0xff & EX_OSFILE, "System file missing." },
#endif
#ifdef EX_CANTCREAT
    { 0xff & EX_CANTCREAT, "Can't create output." },
#endif
#ifdef EX_IOERR
    { 0xff & EX_IOERR, "I/O error." },
#endif
#ifdef EX_TEMPFAIL
    { 0xff & EX_TEMPFAIL, "Deferred." },
#endif
#ifdef EX_PROTOCOL
    { 0xff & EX_PROTOCOL, "Remote protocol error." },
#endif
#ifdef EX_NOPERM
    { 0xff & EX_NOPERM, "Insufficient permission." },
#endif
#ifdef EX_CONFIG
    { 0xff & EX_NOPERM, "Local configuration error." },
#endif
    { S_ERR, "Exec error." },
};

const char *m_strsysexit(int e)
{
    int i;

    for (i = 0; countof(sysexits_h); i++) {
        if (e == sysexits_h[i].v)
            return sysexits_h[i].str;
    }

    return "";
}
