/*
 * Copyright notice from original mutt:
 * Copyright (C) 1996-2002 Michael R. Elkins <me@mutt.org>
 * Copyright (C) 2004 g10 Code GmbH
 *
 * This file is part of mutt-ng, see http://www.muttng.org/.
 * It's licensed under the GNU General Public License,
 * please see the file GPL in the top level source directory.
 */

#ifndef _MUTT_H
#define _MUTT_H

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <stdio.h>
#include <limits.h>
#include <signal.h>

#ifndef _POSIX_PATH_MAX
#include <posix1_lim.h>
#endif

#include <lib-lib/lib-lib.h>

#include <lib-mime/mime.h>

#define MUTT_VERSION (VERSION)

/* nifty trick I stole from ELM 2.5alpha. */
#ifdef MAIN_C
#define WHERE
#define INITVAL(x) = x
#else
#define WHERE extern
#define INITVAL(x)
#endif

typedef struct {
  int ch;                       /* raw key pressed */
  int op;                       /* function op */
} event_t;

/* flags for mutt_FormatString() */
typedef enum {
  M_FORMAT_FORCESUBJ = (1 << 0),        /* print the subject even if unchanged */
  M_FORMAT_TREE = (1 << 1),     /* draw the thread tree */
  M_FORMAT_MAKEPRINT = (1 << 2),        /* make sure that all chars are printable */
  M_FORMAT_OPTIONAL = (1 << 3),
  M_FORMAT_STAT_FILE = (1 << 4),        /* used by mutt_attach_fmt */
  M_FORMAT_ARROWCURSOR = (1 << 5),      /* reserve space for arrow_cursor */
  M_FORMAT_INDEX = (1 << 6)     /* this is a main index entry */
} format_flag;

/* types for mutt_add_hook() */
#define M_FOLDERHOOK	1
#define M_MBOXHOOK	(1<<1)
#define M_SENDHOOK	(1<<2)
#define M_FCCHOOK	(1<<3)
#define M_SAVEHOOK	(1<<4)
#define M_CHARSETHOOK	(1<<5)
#define M_ICONVHOOK 	(1<<6)
#define M_MESSAGEHOOK	(1<<7)
#define M_CRYPTHOOK	(1<<8)
#define M_ACCOUNTHOOK	(1<<9)
#define M_REPLYHOOK	(1<<10)
#define M_SEND2HOOK     (1<<11)

#define M_OPENHOOK	(1<<12)
#define M_APPENDHOOK	(1<<13)
#define M_CLOSEHOOK	(1<<14)

/* tree characters for linearize_tree and print_enriched_string */
#define M_TREE_LLCORNER		1
#define M_TREE_ULCORNER		2
#define M_TREE_LTEE		3
#define M_TREE_HLINE		4
#define M_TREE_VLINE		5
#define M_TREE_SPACE		6
#define M_TREE_RARROW		7
#define M_TREE_STAR		8
#define M_TREE_HIDDEN		9
#define M_TREE_EQUALS		10
#define M_TREE_TTEE		11
#define M_TREE_BTEE		12
#define M_TREE_MISSING		13
#define M_TREE_MAX		14

#define M_THREAD_COLLAPSE	(1<<0)
#define M_THREAD_UNCOLLAPSE	(1<<1)
#define M_THREAD_GET_HIDDEN	(1<<2)
#define M_THREAD_UNREAD		(1<<3)
#define M_THREAD_NEXT_UNREAD	(1<<4)

enum {
  /* modes for mutt_view_attachment() */
  M_REGULAR = 1,
  M_MAILCAP,
  M_AS_TEXT,

  /* action codes used by mutt_set_flag() and mutt_pattern_function() */
  M_ALL,
  M_NONE,
  M_NEW,
  M_OLD,
  M_REPLIED,
  M_READ,
  M_UNREAD,
  M_DELETE,
  M_UNDELETE,
  M_DELETED,
  M_APPENDED,
  M_PURGED,
  M_FLAG,
  M_TAG,
  M_UNTAG,
  M_LIMIT,
  M_EXPIRED,
  M_SUPERSEDED,

  /* actions for mutt_pattern_comp/mutt_pattern_exec */
  M_AND,
  M_OR,
  M_TO,
  M_CC,
  M_COLLAPSED,
  M_SUBJECT,
  M_FROM,
  M_DATE,
  M_DATE_RECEIVED,
  M_DUPLICATED,
  M_UNREFERENCED,
  M_ID,
  M_BODY,
  M_HEADER,
  M_HORMEL,
  M_WHOLE_MSG,
  M_SENDER,
  M_MESSAGE,
  M_SCORE,
  M_SIZE,
  M_REFERENCE,
  M_RECIPIENT,
  M_LIST,
  M_SUBSCRIBED_LIST,
  M_PERSONAL_RECIP,
  M_PERSONAL_FROM,
  M_ADDRESS,
  M_CRYPT_SIGN,
  M_CRYPT_VERIFIED,
  M_CRYPT_ENCRYPT,
  M_PGP_KEY,
  M_XLABEL,
  M_MIMEATTACH,
#ifdef USE_NNTP
  M_NEWSGROUPS,
#endif
  M_REALNAME,
  M_MULTIPART,

  /* Options for Mailcap lookup */
  M_EDIT,
  M_COMPOSE,
  M_PRINT,
  M_AUTOVIEW,

  /* options for socket code */
  M_NEW_SOCKET,
#ifdef USE_SSL
  M_NEW_SSL_SOCKET,
#endif

  /* Options for mutt_save_attachment */
  M_SAVE_APPEND,
  M_SAVE_OVERWRITE
};

/* possible arguments to set_quadoption() */
enum {
  M_NO,
  M_YES,
  M_ASKNO,
  M_ASKYES
};

/* quad-option vars */
enum {
  OPT_ABORT,
  OPT_ATTACH,
  OPT_BOUNCE,
  OPT_COPY,
  OPT_DELETE,
  OPT_FORWEDIT,
  OPT_INCLUDE,
  OPT_IMAPRECONNECT,
  OPT_MFUPTO,
  OPT_MIMEFWD,
  OPT_MIMEFWDREST,
  OPT_MOVE,
  OPT_PGPMIMEAUTO,              /* ask to revert to PGP/MIME when inline fails */
  OPT_POPDELETE,
  OPT_POPRECONNECT,
  OPT_POSTPONE,
  OPT_PRINT,
  OPT_QUIT,
  OPT_REPLYTO,
  OPT_RECALL,
#if defined(USE_SSL) || defined(USE_GNUTLS)
  OPT_SSLSTARTTLS,
#endif
  OPT_SUBJECT,
  OPT_VERIFYSIG,                /* verify PGP signatures */
  OPT_LISTREPLY,
#ifdef USE_NNTP
  OPT_TOMODERATED,
  OPT_NNTPRECONNECT,
  OPT_CATCHUP,
  OPT_FOLLOWUPTOPOSTER,
#endif                          /* USE_NNTP */

  /* THIS MUST BE THE LAST VALUE. */
  OPT_MAX
};

/* flags to ci_send_message() */
#define SENDREPLY	(1<<0)
#define SENDGROUPREPLY	(1<<1)
#define SENDLISTREPLY	(1<<2)
#define SENDFORWARD	(1<<3)
#define SENDPOSTPONED	(1<<4)
#define SENDBATCH	(1<<5)
#define SENDKEY		(1<<6)
#define SENDRESEND	(1<<7)
#define SENDNEWS	(1<<8)

/* flags to _mutt_select_file() */
#define M_SEL_BUFFY	(1<<0)
#define M_SEL_MULTI	(1<<1)
#define M_SEL_FOLDER	(1<<2)

/* flags for parse_spam_list */
#define M_SPAM          1
#define M_NOSPAM        2

/* boolean vars */
enum {
  OPTALLOW8BIT,
  OPTALLOWANSI,
  OPTARROWCURSOR,
  OPTASCIICHARS,
  OPTASKBCC,
  OPTASKCC,
  OPTASKFOLLOWUP,
  OPTASKXCOMMENTTO,
  OPTATTACHSPLIT,
  OPTAUTOEDIT,
  OPTAUTOTAG,
  OPTBEEP,
  OPTBEEPNEW,
  OPTBOUNCEDELIVERED,
  OPTBRAILLEFRIENDLY,
  OPTCHECKNEW,
  OPTCOLLAPSEUNREAD,
  OPTCONFIRMAPPEND,
  OPTCONFIRMCREATE,
  OPTCOUNTATTACH,
  OPTDELETEUNTAG,
  OPTDELSP,
  OPTDIGESTCOLLAPSE,
  OPTDUPTHREADS,
  OPTEDITHDRS,
  OPTENCODEFROM,
  OPTENVFROM,
  OPTFASTREPLY,
  OPTFCCATTACH,
  OPTFCCCLEAR,
  OPTFOLLOWUPTO,
  OPTFORCEBUFFYCHECK,
  OPTFORCENAME,
  OPTFORWDECODE,
  OPTFORWQUOTE,
#ifdef USE_HCACHE
  OPTHCACHEVERIFY,
#ifdef HAVE_QDBM
  OPTHCACHECOMPRESS,
#endif /* HAVE_QDBM */
#endif
  OPTHDRS,
  OPTHEADER,
  OPTHELP,
  OPTHIDDENHOST,
  OPTHIDELIMITED,
  OPTHIDEMISSING,
  OPTHIDETHREADSUBJECT,
  OPTHIDETOPLIMITED,
  OPTHIDETOPMISSING,
  OPTIGNORELISTREPLYTO,
  OPTIMAPCHECKSUBSCRIBED,
  OPTIMAPLSUB,
  OPTIMAPPASSIVE,
  OPTIMAPPEEK,
  OPTIMAPSERVERNOISE,
#if defined(USE_SSL) || defined(USE_GNUTLS)
# ifndef USE_GNUTLS
  OPTSSLSYSTEMCERTS,
  OPTSSLV2,
# endif /* !USE_GNUTLS */
  OPTSSLV3,
  OPTTLSV1,
  OPTSSLFORCETLS,
#endif /* USE_SSL || USE_GNUTLS */
  OPTIMPLICITAUTOVIEW,
  OPTINCLUDEONLYFIRST,
  OPTKEEPFLAGGED,
  OPTMAILCAPSANITIZE,
  OPTMAILDIRTRASH,
  OPTMARKERS,
  OPTMARKOLD,
  OPTMBOXPANE,
  OPTMENUSCROLL,                /* scroll menu instead of implicit next-page */
  OPTMENUMOVEOFF,
  OPTMETAKEY,                   /* interpret ALT-x as ESC-x */
  OPTMETOO,
  OPTMHPURGE,
  OPTMIMEFORWDECODE,
#ifdef USE_NNTP
  OPTMIMESUBJECT,               /* encode subject line with RFC2047 */
#endif
  OPTNARROWTREE,
  OPTPAGERSTOP,
  OPTPIPEDECODE,
  OPTPIPESPLIT,
  OPTPOPAUTHTRYALL,
  OPTPOPLAST,
  OPTPRINTDECODE,
  OPTPRINTSPLIT,
  OPTPROMPTAFTER,
  OPTQUOTEEMPTY,
  OPTQUOTEQUOTED,
  OPTREADONLY,
  OPTREPLYSELF,
  OPTRESOLVE,
  OPTREVALIAS,
  OPTREVNAME,
  OPTREVREAL,
  OPTRFC2047PARAMS,
  OPTSAVEADDRESS,
  OPTSAVEEMPTY,
  OPTSAVENAME,
  OPTSCORE,
  OPTSIGDASHES,
  OPTSIGONTOP,
  OPTSORTRE,
  OPTSPAMSEP,
  OPTSTATUSONTOP,
  OPTSTRICTMAILTO,
  OPTSTRICTMIME,
  OPTSTRICTTHREADS,
  OPTSTRIPWAS,
  OPTSTUFFQUOTED,
  OPTSUSPEND,
  OPTTEXTFLOWED,
  OPTTHOROUGHSRC,
  OPTTHREADRECEIVED,
  OPTTILDE,
  OPTUNCOLLAPSEJUMP,
  OPTUSE8BITMIME,
  OPTUSEDOMAIN,
  OPTUSEFROM,
  OPTUSEGPGAGENT,
#ifdef HAVE_LIBIDN
  OPTUSEIDN,
#endif
#ifdef HAVE_GETADDRINFO
  OPTUSEIPV6,
#endif
  OPTWAITKEY,
  OPTWEED,
  OPTWRAP,
  OPTWRAPSEARCH,
  OPTWRITEBCC,                  /* write out a bcc header? */
  OPTXMAILER,
  OPTXMAILTO,                   /* 1 if $edit_header is forcebly set */
  OPTXTERMSETTITLES,

  OPTCRYPTUSEGPGME,

  /* PGP options */

  OPTCRYPTAUTOSIGN,
  OPTCRYPTAUTOENCRYPT,
  OPTCRYPTAUTOPGP,
  OPTCRYPTAUTOSMIME,
  OPTCRYPTREPLYENCRYPT,
  OPTCRYPTREPLYSIGN,
  OPTCRYPTREPLYSIGNENCRYPTED,
  OPTCRYPTTIMESTAMP,
  OPTSMIMEISDEFAULT,
  OPTASKCERTLABEL,
  OPTSDEFAULTDECRYPTKEY,
  OPTPGPIGNORESUB,
  OPTPGPCHECKEXIT,
  OPTPGPLONGIDS,
  OPTPGPAUTODEC,
#if 0
  OPTPGPENCRYPTSELF,
#endif
  OPTPGPRETAINABLESIG,
  OPTPGPSTRICTENC,
  OPTFORWDECRYPT,
  OPTPGPSHOWUNUSABLE,
  OPTPGPAUTOINLINE,
  OPTPGPREPLYINLINE,

  /* news options */

#ifdef USE_NNTP
  OPTSHOWNEWNEWS,
  OPTSHOWONLYUNREAD,
  OPTSAVEUNSUB,
  OPTLOADDESC,
  OPTXCOMMENTTO,
#endif                          /* USE_NNTP */

  /* pseudo options */

  OPTAUXSORT,                   /* (pseudo) using auxillary sort function */
  OPTFORCEREFRESH,              /* (pseudo) refresh even during macros */
  OPTNOCURSES,                  /* (pseudo) when sending in batch mode */
  OPTNEEDREDRAW,                /* (pseudo) to notify caller of a submenu */
  OPTSEARCHREVERSE,             /* (pseudo) used by ci_search_command */
  OPTMSGERR,                    /* (pseudo) used by mutt_error/mutt_message */
  OPTSEARCHINVALID,             /* (pseudo) used to invalidate the search pat */
  OPTSIGNALSBLOCKED,            /* (pseudo) using by mutt_block_signals () */
  OPTSYSSIGNALSBLOCKED,         /* (pseudo) using by mutt_block_signals_system () */
  OPTNEEDRESORT,                /* (pseudo) used to force a re-sort */
  OPTRESORTINIT,                /* (pseudo) used to force the next resort to be from scratch */
  OPTVIEWATTACH,                /* (pseudo) signals that we are viewing attachments */
  OPTFORCEREDRAWINDEX,          /* (pseudo) used to force a redraw in the main index */
  OPTFORCEREDRAWPAGER,          /* (pseudo) used to force a redraw in the pager */
  OPTSORTSUBTHREADS,            /* (pseudo) used when $sort_aux changes */
  OPTNEEDRESCORE,               /* (pseudo) set when the `score' command is used */
  OPTATTACHMSG,                 /* (pseudo) used by attach-message */
  OPTHIDEREAD,                  /* (pseudo) whether or not hide read messages */
  OPTKEEPQUIET,                 /* (pseudo) shut up the message and refresh
                                 *          functions while we are executing an
                                 *          external program.
                                 */
  OPTMENUCALLER,                /* (pseudo) tell menu to give caller a take */
  OPTREDRAWTREE,                /* (pseudo) redraw the thread tree */
  OPTPGPCHECKTRUST,             /* (pseudo) used by pgp_select_key () */
  OPTDONTHANDLEPGPKEYS,         /* (pseudo) used to extract PGP keys */
  OPTUNBUFFEREDINPUT,           /* (pseudo) don't use key buffer */

#ifdef USE_NNTP
  OPTNEWS,                      /* (pseudo) used to change reader mode */
  OPTNEWSSEND,                  /* (pseudo) used to change behavior when posting */
  OPTNEWSCACHE,                 /* (pseudo) used to indicate if news cache exist */
#endif
  OPTSHORTENHIERARCHY,          /* set when to shorten "hierarchies" in the sidebar */
  OPTSIDEBARNEWMAILONLY,
  OPTMAX
};

#define mutt_bit_alloc(n) p_new(char, (n + 7) / 8)
#define mutt_bit_set(v,n) v[n/8] |= (1 << (n % 8))
#define mutt_bit_unset(v,n) v[n/8] &= ~(1 << (n % 8))
#define mutt_bit_toggle(v,n) v[n/8] ^= (1 << (n % 8))
#define mutt_bit_isset(v,n) (v[n/8] & (1 << (n % 8)))

#define set_option(x) mutt_bit_set(Options,x)
#define unset_option(x) mutt_bit_unset(Options,x)
#define toggle_option(x) mutt_bit_toggle(Options,x)
#define option(x) mutt_bit_isset(Options,x)

typedef struct spam_list_t {
  rx_t *rx;
  int nmatch;
  char *template;
  struct spam_list_t *next;
} SPAM_LIST;


#define mutt_new_spam_list() p_new(SPAM_LIST, 1)
void mutt_free_spam_list (SPAM_LIST **);

int mutt_matches_ignore (const char *, string_list_t *);

void mutt_init (int, string_list_t *);

typedef struct thread {
  unsigned int fake_thread:1;
  unsigned int duplicate_thread:1;
  unsigned int sort_children:1;
  unsigned int check_subject:1;
  unsigned int visible:1;
  unsigned int deep:1;
  unsigned int subtree_visible:2;
  unsigned int next_subtree_visible:1;
  struct thread *parent;
  struct thread *child;
  struct thread *next;
  struct thread *prev;
  HEADER *message;
  HEADER *sort_key;
} THREAD;


/* flag to mutt_pattern_comp() */
#define M_FULL_MSG      (1<<0)       /* enable body and header matching */

typedef enum {
  M_MATCH_FULL_ADDRESS = 1
} pattern_exec_flag;

typedef struct pattern_t {
  short op;
  unsigned int not : 1;
  unsigned int alladdr : 1;
  unsigned int stringmatch : 1;
  int min;
  int max;
  struct pattern_t *next;
  struct pattern_t *child;      /* arguments to logical op */
  char* str;
  regex_t *rx;
} pattern_t;

typedef struct {
  char *path;
  FILE *fp;
  time_t mtime;
  time_t mtime_cur;             /* used with maildir folders */
  off_t size;
  off_t vsize;
  char *pattern;                /* limit pattern string */
  pattern_t *limit_pattern;     /* compiled limit pattern */
  HEADER **hdrs;
  HEADER *last_tag;             /* last tagged msg. used to link threads */
  THREAD *tree;                 /* top of thread tree */
  HASH *id_hash;                /* hash table by msg id */
  HASH *subj_hash;              /* hash table by subject */
  HASH *thread_hash;            /* hash table for threading */
  int *v2r;                     /* mapping from virtual to real msgno */
  int hdrmax;                   /* number of pointers in hdrs */
  int msgcount;                 /* number of messages in the mailbox */
  int vcount;                   /* the number of virtual messages */
  int tagged;                   /* how many messages are tagged? */
  int new;                      /* how many new messages? */
  int unread;                   /* how many unread messages? */
  int deleted;                  /* how many deleted messages */
  int appended;                 /* how many saved messages? */
  int flagged;                  /* how many flagged messages */
  int msgnotreadyet;            /* which msg "new" in pager, -1 if none */
  void *data;                   /* driver specific data */

  short magic;                  /* mailbox type */

  void *compressinfo;           /* compressed mbox module private data */
  char *realpath;               /* path to compressed mailbox */

  unsigned int locked:1;        /* is the mailbox locked? */
  unsigned int changed:1;       /* mailbox has been modified */
  unsigned int readonly:1;      /* don't allow changes to the mailbox */
  unsigned int dontwrite:1;     /* dont write the mailbox on close */
  unsigned int append:1;        /* mailbox is opened in append mode */
  unsigned int quiet:1;         /* inhibit status messages? */
  unsigned int collapsed:1;     /* are all threads collapsed? */
  unsigned int closing:1;       /* mailbox is being closed */
  unsigned int counting:1;      /* do we just want to cound? */
} CONTEXT;

/* for attachment counter */
typedef struct {
  char *major;
  int major_int;
  const char *minor;
  regex_t minor_rx;
} ATTACH_MATCH;

/* Flags for mutt_count_body_parts() */
#define M_PARTS_TOPLEVEL (1<<0) /* is the top-level part */
#define M_PARTS_RECOUNT (1<<1) /* force recount */

#include "protos.h"
#include "globals.h"

#endif /* !_MUTT_H */
