/*
 * Copyright notice from original mutt:
 * Copyright (C) 1996-2000 Michael R. Elkins <me@mutt.org>
 *
 * This file is part of mutt-ng, see http://www.muttng.org/.
 * It's licensed under the GNU General Public License,
 * please see the file GPL in the top level source directory.
 */

#ifdef HAVE_CONFIG_H
# include "config.h"
#endif

#include <sys/stat.h>

#if HAVE_STDINT_H
#include <stdint.h>
#elif HAVE_INTTYPES_H
#include <inttypes.h>
#endif

#define MoreArgs(p) (*p->dptr && *p->dptr != ';' && *p->dptr != '#')

typedef const char *format_t (char *, ssize_t, char, const char *,
                              const char *, const char *, const char *,
                              unsigned long, format_flag);

void mutt_FormatString (char *, ssize_t, const char *, format_t *,
                        unsigned long, format_flag);

void set_quadoption (int, int);
int query_quadoption (int, const char *);
int quadoption (int);

int mutt_option_value (const char* val, char* dst, ssize_t dstlen);

address_t *mutt_default_from (void);

BODY *mutt_make_file_attach (const char *);
BODY *mutt_make_message_attach (CONTEXT *, HEADER *, int);
BODY *mutt_remove_multipart (BODY *);
BODY *mutt_make_multipart (BODY *);

CONTENT *mutt_get_content_info (const char *fname, BODY * b);

int mutt_cmp_header (const HEADER*, const HEADER*);
int mutt_cmp_addr (const address_t * a, const address_t * b);
int mutt_cmp_list (const string_list_t * a, const string_list_t * b);
int mutt_cmp_env (const ENVELOPE * e1, const ENVELOPE * e2);
int mutt_cmp_body (const BODY * b1, const BODY * b2);

time_t mutt_decrease_mtime (const char *, struct stat *);
void mutt_set_mtime (const char*, const char*);
int is_from (const char *, char *, ssize_t, time_t *);

const char *mutt_charset_hook (const char *);
const char *mutt_iconv_hook (const char *);
char *mutt_expand_path (char *, ssize_t);
char *_mutt_expand_path (char *, ssize_t, int);
char *mutt_find_hook (int, const char *);
char *mutt_gen_msgid (void);
char *mutt_get_body_charset (char *, ssize_t, BODY *);
const char *mutt_get_name (address_t *);
const char *mutt_crypt_hook (address_t *);
char *mutt_make_date (char *, ssize_t);

const char *mutt_make_version (int full);

const char *mutt_fqdn (short);

void mutt_account_hook (const char *url);
void mutt_add_to_reference_headers (ENVELOPE * env, ENVELOPE * curenv,
                                    string_list_t *** pp, string_list_t *** qq);
void mutt_adv_mktemp (const char*, char*, ssize_t);
int mutt_bounce_message (FILE * fp, HEADER *, address_t *);
void mutt_check_rescore (CONTEXT *);
void mutt_default_save (char *, ssize_t, HEADER *);
void mutt_display_address (ENVELOPE *);
void mutt_edit_content_type (HEADER *, BODY *, FILE *);
void mutt_edit_headers (const char *, const char *, HEADER *, char *, ssize_t);
void mutt_enter_command (void);
void mutt_expand_file_fmt (char *, ssize_t, const char *, const char *);
void mutt_expand_fmt (char *, ssize_t, const char *, const char *);
void mutt_expand_link (char *, const char *, const char *);
void mutt_fix_reply_recipients (ENVELOPE * env);
void mutt_folder_hook (char *);
void mutt_forward_intro (FILE * fp, HEADER * cur);
void mutt_forward_trailer (FILE * fp);
void mutt_help (int);
void mutt_draw_tree (CONTEXT *);
void mutt_make_attribution (CONTEXT * ctx, HEADER * cur, FILE * out);
void mutt_make_forward_subject (ENVELOPE * env, CONTEXT * ctx, HEADER * cur);
void mutt_make_help (char *, ssize_t, char *, int, int);
char *mutt_compile_help(char *, ssize_t, int, struct mapping_t *);
void mutt_make_misc_reply_headers (ENVELOPE * env, CONTEXT * ctx,
                                   HEADER * cur, ENVELOPE * curenv);
void mutt_make_post_indent (CONTEXT * ctx, HEADER * cur, FILE * out);
void mutt_merge_envelopes (ENVELOPE* base, ENVELOPE** extra);
void mutt_message_to_7bit (BODY *, FILE *);

#define mutt_mktemp(a) _mutt_mktemp (a, __FILE__, __LINE__)
void _mutt_mktemp (char *, const char *, int);
void mutt_prepare_envelope (ENVELOPE *, int);
void mutt_unprepare_envelope (ENVELOPE *);
void mutt_pretty_mailbox (char *);
void mutt_pretty_size (char *, ssize_t, long);
void mutt_pipe_message (HEADER *);
void mutt_print_message (HEADER *);
void mutt_safe_path (char *s, ssize_t l, address_t * a);
void mutt_save_path (char *s, ssize_t l, address_t * a);
void mutt_score_message (CONTEXT *, HEADER *, int);
void mutt_select_fcc (char *, ssize_t, HEADER *);

#define mutt_select_file(A,B,C) _mutt_select_file(A,B,C,NULL,NULL)
void _mutt_select_file (char *, ssize_t, int, char ***, int *);
void mutt_message_hook (CONTEXT *, HEADER *, int);
void _mutt_set_flag (CONTEXT *, HEADER *, int, int, int);

#define mutt_set_flag(a,b,c,d) _mutt_set_flag(a,b,c,d,1)
void mutt_set_followup_to (ENVELOPE *);
void mutt_shell_escape (void);
void mutt_stamp_attachment (BODY * a);
void mutt_tag_set_flag (int, int);
void mutt_update_encoding (BODY * a);
void mutt_version (void);
void mutt_write_references (string_list_t *, FILE *);
void mutt_write_address_list (address_t * adr, FILE * fp, int linelen,
                              int display);

int mutt_can_decode (BODY *);
int mutt_change_flag (HEADER *, int);
int mutt_check_menu (const char *);
int mutt_check_month (const char *);
int mutt_check_overwrite (const char *, const char *, char *, ssize_t, int *,
                          char **);
int mutt_check_traditional_pgp (HEADER *, int *);
int mutt_command_complete (char *, ssize_t, int, int);
int mutt_var_value_complete (char *, ssize_t, int);
int mutt_copy_body (FILE *, BODY **, BODY *);

int mutt_display_message (HEADER * h);
int mutt_dump_variables (int full);
int mutt_edit_message (CONTEXT *, HEADER *);
int mutt_fetch_recips (ENVELOPE * out, ENVELOPE * in, int flags);
int mutt_prepare_template (FILE *, CONTEXT *, HEADER *, HEADER *, short);
int mutt_resend_message (FILE *, CONTEXT *, HEADER *);

int mutt_get_hook_type (const char *);
int mutt_get_postponed (CONTEXT *, HEADER *, HEADER **, char *, ssize_t);
int mutt_invoke_mta (address_t *, address_t *, address_t *, address_t *, const char *,
                     int);
int mutt_is_list_cc (int, address_t *, address_t *);
int mutt_is_list_recipient (int, address_t *, address_t *);
int mutt_lookup_mime_type (BODY *, const char *);
int mutt_match_spam_list (const char *, SPAM_LIST *, char *, int);
int mutt_num_postponed (int);
int mutt_parse_bind (BUFFER *, BUFFER *, unsigned long, BUFFER *);
int mutt_parse_exec (BUFFER *, BUFFER *, unsigned long, BUFFER *);
int mutt_parse_hook (BUFFER *, BUFFER *, unsigned long, BUFFER *);
int mutt_parse_macro (BUFFER *, BUFFER *, unsigned long, BUFFER *);
int mutt_parse_push (BUFFER *, BUFFER *, unsigned long, BUFFER *);
int mutt_parse_rc_line ( /* const */ char *, BUFFER *, BUFFER *);
int mutt_parse_score (BUFFER *, BUFFER *, unsigned long, BUFFER *);
int mutt_parse_unscore (BUFFER *, BUFFER *, unsigned long, BUFFER *);
int mutt_parse_unhook (BUFFER *, BUFFER *, unsigned long, BUFFER *);
int mutt_pattern_func (int, char *);
int mutt_query_variables (string_list_t * queries);
int _mutt_save_message (HEADER *, CONTEXT *, int, int, int);
int mutt_save_message (HEADER *, int, int, int, int *);
int mutt_search_command (int, int);
int mutt_compose_menu (HEADER *, char *, ssize_t, HEADER *);
int mutt_thread_set_flag (HEADER *, int, int, int);
void mutt_update_num_postponed (void);
int mutt_which_case (const char *);
int mutt_write_fcc (const char *path, HEADER * hdr, const char *msgid, int,
                    char *);
int mutt_write_mime_body (BODY *, FILE *);
int mutt_write_mime_header (BODY *, FILE *);
int mutt_write_rfc822_header (FILE *, ENVELOPE *, BODY *, int, int);
void mutt_sleep (short);
int mutt_save_confirm (const char *, struct stat *);

/* base64.c */
void mutt_to_base64 (unsigned char *, const unsigned char *, ssize_t, ssize_t);
int mutt_from_base64 (char *, const char *);

#define new_pattern() p_new(pattern_t, 1)
#define mutt_new_enter_state() p_new(ENTER_STATE, 1)

int mutt_pattern_exec (struct pattern_t *pat, pattern_exec_flag flags,
                       CONTEXT * ctx, HEADER * h);
pattern_t *mutt_pattern_comp ( /* const */ char *s, int flags, BUFFER * err);
void mutt_check_simple (char *s, ssize_t len, const char *simple);
void mutt_pattern_free (pattern_t ** pat);

void ci_bounce_message (HEADER *, int *);
int ci_send_message (int, HEADER *, char *, CONTEXT *, HEADER *);

/* flags for mutt_extract_token() */
#define M_TOKEN_EQUAL		1       /* treat '=' as a special */
#define M_TOKEN_CONDENSE	(1<<1)  /* ^(char) to control chars (macros) */
#define M_TOKEN_SPACE		(1<<2)  /* don't treat whitespace as a term */
#define M_TOKEN_QUOTE		(1<<3)  /* don't interpret quotes */
#define M_TOKEN_PATTERN		(1<<4)  /* !)|~ are terms (for patterns) */
#define M_TOKEN_COMMENT		(1<<5)  /* don't reap comments */
#define M_TOKEN_SEMICOLON	(1<<6)  /* don't treat ; as special */

int mutt_extract_token(BUFFER *, BUFFER *, int);

